/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef _FOC_KEYS_H
#define _FOC_KEYS_H
/*!
* @file foc_keys.h
*
* @brief This file provides keys control functions interface.
*
*/

/* ===========================================  Includes  =========================================== */
#include <stdint.h>
#include "ac780x_timer_reg.h"
#include "ac780x_gpio.h"

/* ============================================  Define  ============================================ */
#define MOTORAPP_MAIN_VER           1                   /*!< Main version of MotorApp code project */
#define MOTORAPP_SUB_VER            2                   /*!< Sub-version of MotorApp code project */
#define MOTORAPP_REV_VER            6                   /*!< Rev-version of MotorApp code project */

#define MOTORAPP_VER_YEAR           2023                /*!< Year of the current MotorApp code version released */
#define MOTORAPP_VER_MONTH          4                   /*!< Month of the current MotorApp code version released */
#define MOTORAPP_VER_DAY            27                  /*!< Date of the current MotorApp code version released */

#define MASTER                      0                   /*!< Master */
#define MCU_TRUNK_AC780X            (0x10)              /*!< MCU motor code demo branch */
#define MCU_TRUNK_AC780X_HV         (0x11)              /*!< MCU motor code high voltage branch */
#define MCU_TRUNK_AC780X_FAN        (0x12)              /*!< MCU motor code fan branch */
#define MCU_TRUNK_AC780X_HALL       (0x13)              /*!< MCU motor code hall branch */
#define MCU_TRUNK_AC780X_ONE_SHUNT  (0x14)              /*!< MCU motor code one_shunt branch */

#define MOTORAPP_BRANCH             MCU_TRUNK_AC780X    /*!< Code branch */

/*!
* @brief Panle digital display driver.
*/
#define LED_SIN_165_IO      GPIO_GetPinLevel(GPIOB, GPIO_PIN15)     /*!< Get GPIO PB-15 level */

#define LED_CLK_IO_0        GPIO_SetPinBit(GPIOB, GPIO_PIN14)       /*!< Set GPIO PB-14 bit */
#define LED_CLK_IO_1        GPIO_ResetPinBit(GPIOB, GPIO_PIN14)     /*!< Reset GPIO PB-14 bit */

#define LED_SOUT_595_IO_0   GPIO_SetPinBit(GPIOB, GPIO_PIN3)        /*!< Set GPIO PB-3 bit */
#define LED_SOUT_595_IO_1   GPIO_ResetPinBit(GPIOB, GPIO_PIN3)      /*!< Reset GPIO PB-3 bit */

#define LED_165_SHL_IO_1    GPIO_SetPinBit(GPIOA, GPIO_PIN4)        /*!< Set GPIO PA-4 bit */
#define LED_165_SHL_IO_0    GPIO_ResetPinBit(GPIOA, GPIO_PIN4)      /*!< Reset GPIO PA-4 bit */

#define MC_IO_1             GPIO_SetPinBit(GPIOB, GPIO_PIN2)        /*!< Set GPIO PB-2 bit */
#define MC_IO_0             GPIO_ResetPinBit(GPIOB, GPIO_PIN2)      /*!< Reset GPIO PB-2 bit */

#define LED0                0xC0
#define LED1                0xF9
#define LED2                0xA4
#define LED3                0xB0
#define LED4                0x99
#define LED5                0x92
#define LED6                0x82
#define LED7                0xF8
#define LED8                0x80
#define LED9                0x90
#define LED0P               0x40
#define LED1P               0x79
#define LED2P               0x24
#define LED3P               0x30
#define LED4P               0x19
#define LED5P               0x12
#define LED6P               0x02
#define LED7P               0x78
#define LED8P               0x00
#define LED9P               0x10
#define LED_A               0x88
#define LED_B               0x83
#define LED_C               0xC6
#define LED_D               0xa1
#define LED_E               0x86
#define LED_F               0x8E
#define LED_G               0xc2
#define LED_H               0x89
#define LED_I               0xfb
#define LED_J               0xf1
#define LED_L               0xc7
#define LED_M               0xaa
#define LED_N               0xab
#define LED_O               0xa3
#define LED_P               0x8C
#define LED_R               0xaf
#define LED_S               0x92
#define LED_T               0x87
#define LED_U               0xe3
//#define LED_V               0xe2
#define LED_V               0xc1
#define LED_Y               0x91
#define LED_AP              0x08
#define LED_BP              0x03
#define LED_CP              0x46
#define LED_DP              0x21
#define LED_EP              0x06
#define LED_FP              0x0E
#define LED_GP              0x42
#define LED_HP              0x09
#define LED_IP              0x7b
#define LED_JP              0x71
#define LED_LP              0x47
#define LED_MP              0x2a
#define LED_NP              0x2b
#define LED_OP              0x23
#define LED_PP              0x0C
#define LED_RP              0x2f
#define LED_SP              0x12
#define LED_TP              0x07
#define LED_UP              0x63
#define LED_VP              0x62
#define LED_YP              0x11
#define LEDBAR              0xBF
#define LEDBL               0xFF

/*!
* @brief Panel keys address.
*/
#define MEMU_KEY            0X00FF
#define PLUS_KEY            0X8000
#define SHIFT_KEY           0X1000
#define STARTSTOP_KEY       0X2000
#define MINUS_KEY           0X0200
#define CONFIRM_KEY         0X0800

/* ===========================================  Typedef  ============================================ */
/*!
* @brief Define motor control status enumeration.
*/
typedef enum
{
    POWER_ON            =   0,          /*!< Power on, enter this state after the control board is powered on */
    IDLE                =   1,          /*!< Idle( stand by), enter this state after POWER_ON and the parameter initialization is finished */
    START               =   2,          /*!< Start, if not in TAIL_HEAD_WIND mode, enter this state after the Start key is pressed */
    ANGLE_JUDGE         =   3,          /*!< Angle judge, enter this state after Start key is pressed, use PULSE_INJECTION function to get motor initial position */
    SPEED_JUDGE         =   4,          /*!< Speed judge, enter this state after Start key is pressed in TAIL_HEAD_WIND mode */
    FLY_RUN             =   5,          /*!< Fly run, enter this state after SPEED_JUDGE is complete in TAIL_HEAD_WIND mode */
    START_BRAKE         =   6,          /*!< Start brake, enter this state when the brake function is required. */
    ALIGN               =   7,          /*!< Align, enter this state when the initial angle positioning needs to be performed */
    RUN                 =   8,          /*!< Run, the control system enters this state and performs motor drive control */
    STOP                =   9,          /*!< Stop, the control system stops driving the motor and enters this state */
    SLOW_DOWN           =   10,         /*!< Slow down, this state is entered when the deceleration stop mode(Slow down) is selected and the stop command is executed */
    STOP_BRAKE          =   11,         /*!< Stop brake, enter this state when the brake function is required. */
    IDENTIFY            =   12,         /*!< Identify, enter this state and start motor parameters identification */
    FAULT               =   13,         /*!< Fault, this state is entered when the control system is running fault */
    RESTART             =   14,         /*!< Restart, this state is entered when the startup error happens */
    FLASH_TASK          =   15,         /*!< Flash task, this state is entered when master commands flash operation */
    HALL_SELFLEARN      =   16,         /*!< Hall sensor self learn, this state is entered when needed to obtain Hall sensor installation information */
    INIT_POS            =   17,         /*!< Initial position check, inject rotation high frequency voltage to get rotor position angle */
    TEMPORARY_BUFFER    =   18,         /*!< Temporary buffer, this state is a temporary buffer state between state machine transition */
} MC_STATUS;

/*!
* @brief Panel keys structure.
*/
typedef struct {
    uint8_t memuKey;                    /*!< Panel menu key */
    uint8_t plusKey;                    /*!< Panel plus key */
    uint8_t shiftKey;                   /*!< Panel shift key */
    uint8_t startStopKey;               /*!< Panel start/stop key */
    uint8_t minusKey;                   /*!< Panel reduce key */
    uint8_t confirmKey;                 /*!< Panel confirm key */
} KEY_STATUS;

/* ==========================================  Variables  =========================================== */
extern MC_STATUS g_mcStatus;            /*!< Define motor operation status */
extern MC_STATUS g_mcBufStatus;         /*!< define motor operation buffer status */

static uint8_t s_keyStartPressed = 0;   /*!< Used to judge whether the Start/Stop key acts */
static uint8_t s_keyDirPressed = 0;     /*!< Used to judge whether the Forward/Reverse key acts */
static uint8_t s_keyUpPressed = 0;      /*!< Used to judge whether the Fast key acts */
static uint8_t s_keyDownPressed = 0;    /*!< Used to judge whether the Slow key acts */

extern uint16_t DSP2[6];                /*!< panel display */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief Control panel refresh. Must refresh periodically.
*
* @param[in] none
* @return none
*/
void Panel_Refresh(void);

/*!
* @brief Control panel key GPIO configuration.
*
* @param[in] none
* @return none
*/
void Panel_Config(void);

/*!
* @brief Get the response of pressing each key on the control board.
*
* @param[in] none
* @return none
*/
void MC_Keys_Read(void);

/*!
* @brief Get the main status of motor control right now.
*
* @param[in] none
* @return MC_STATUS: current motor control main status
*/
MC_STATUS Get_McStatus(void);

/*!
* @brief Set the main status of motor control right now.
*
* @param[in] MC_STATUS: status to beset
* @return none
*/
void Set_McStatus(MC_STATUS status);

/*!
* @brief Push the buffer status in motor control main status buffer.
*
* @param[in] MC_STATUS: statusBuffer to store
* @return none
*/
void PushIn_McBufStatus(MC_STATUS statusBuffer);

/*!
* @brief Pull out buffer status to refresh motor control main status.
*
* @param[in] none
* @return none
*/
void PullOut_McBufStatus(void);

/* ======================================  Functions define  ======================================== */

#endif /* _FOC_KEYS_H */

/* =============================================  EOF  ============================================== */
