/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __FOC_ISR_H__
#define __FOC_ISR_H__
/*!
* @file foc_isr.h
*
* @brief This file provides FOC control interrupt callback functions interface.
*
*/

/* ===========================================  Includes  =========================================== */
#include <stdint.h>

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief PWMx Fault Test CallBack Function. When hardware over-current fault occurs, PWM fault interrupt is triggered and this function is called.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void FOC_PWM_FaultCallback(void *device, uint32_t wpara, uint32_t lpara);

/*!
* @brief Callback routine of TIMER0. The interruption period is 1ms. The Task scheduler update in Timer interrupt.
*        When the Timer_0 counts to 1ms, the Timer_0 interrupt is triggered and this function is called.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void Timer0_Callback(void *device, uint32_t wpara, uint32_t lpara);

/*!
* @brief Callback routine of TIMER1. The interruption period is M_T_PERIOD. The Task scheduler update in Timer interrupt.
*        When the Timer_1 counts to M_T_PERIOD, the Timer_1 interrupt is triggered and this function is called.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void Timer1_Callback(void *device, uint32_t wpara, uint32_t lpara);

/*!
* @brief Call back function of PWDT modular. The PWDT modular is mainly used by hall sensor.
*        When PWDT modular input signal level change or PWDT timer overflow will trriger PWDT interrupt.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void FOC_PWDT_Callback(void *device, uint32_t wpara, uint32_t lpara);

/*!
* @brief PWM count overflow interrupt to statistic circles for encoder position servo applicaton.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void FOC_Encoder_Callback(void *device, uint32_t wpara, uint32_t lpara);

/*!
* @brief Encoder Z single gpio interrupt to fix error for ABZ encoder applicaton.
*
* @param[in] device: device pointer
* @param[in] wpara: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void GPIO_EncoderZ_callback(void *device, uint32_t wpara, uint32_t lpara);

/*!
* @brief ADC interrupt service for FOC control.
*
* @param[in] device: device
* @param[in] wparam: callback parameter
* @param[in] lpara: callback parameter
* @return none
*/
void Motor_Ctrl_CallBack(void *device, uint32_t wpara, uint32_t lpara);

/*!
* @brief ADC interrupt service subfunction for short brake.
*        When motor is short circuit braking, ADC interruption will enter this function.
*
* @param[in] none
* @return none
*/
void ShortBrake_AdcProcess(void);

/* ======================================  Functions define  ======================================== */

#endif /* __FOC_ISR_H__ */

/* =============================================  EOF  ============================================== */
