/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file foc_mcu_config.c
*
* @brief This file provides MCU peripheral initialization functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "ac780x.h"
#include "ac780x_gpio_reg.h"
#include "ac780x_pwdt_reg.h"
#include "ac780x_adc_reg.h"
#include "ac780x_ctu_reg.h"
#include "ac780x_pwm_reg.h"
//#include "ac780x_acmp_reg.h"
#include "ac780x_dma_reg.h"
#include "ac780x_spi_reg.h"
#include "globalvar.h"
#include "foc_isr.h"
#include "foc_app.h"
#include "foc_mcu_config.h"
#include "foc_keys.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
uint32_t g_ADCRegularBuffer[10] = {0};  /*!< Buffer to storage Regular Group ADC Value */

/* spi Master state */
SPI_ConfigType spiMasteConfig;

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief FOC Control Module GPIO Initialize.
*
* @param[in] none
* @return none
*/
void FOC_GPIO_Initialize(void)
{
//#ifdef USE_GPIO_CALCULATE_FOC_RUNTIME
    /* Setting the multiplexing function of GPIO pin */
    GPIO_SetFunc(GPIOC, GPIO_PIN9, GPIO_FUN0);
    /* Setting output as the attribute of GPIO pin */
    GPIO_SetDir(GPIOC, GPIO_PIN9, GPIO_OUT);
//#endif

    /* Setting the LED2 to display operation states */
    GPIO_SetFunc(GPIOA, GPIO_PIN6, GPIO_FUN0);
    GPIO_SetDir(GPIOA, GPIO_PIN6, GPIO_OUT);
}

/*!
* @brief FOC Control Module ACMP Initialize.
*
* @param[in] none
* @return none
*/
//void FOC_ACMP_Initialize(void)
//{
//    ACMP_ConfigType acmpConfig;
//
//    /*
//    * GPIO Configuration for ACMP
//    * PA11    --> ADC_IN0
//    * PA10    --> ADC_IN1
//    * PA9     --> ADC_IN2
//    * PA8     --> ADC_IN3
//    */
//    /* ADC channel 0, Bemf_U */
//    GPIO_SetFunc(GPIOA, GPIO_PIN11, GPIO_FUN2);
//    /* ADC channel 1, Bemf_V */
//    GPIO_SetFunc(GPIOA, GPIO_PIN10, GPIO_FUN2);
//    /* ADC channel 2, Bemf_W */
//    GPIO_SetFunc(GPIOA, GPIO_PIN9, GPIO_FUN2);
//    /* ADC channel 3, Bemf_ref */
//    GPIO_SetFunc(GPIOA, GPIO_PIN8, GPIO_FUN2);
//
//    memset(&acmpConfig, 0, sizeof(ACMP_ConfigType));
//
//    /* ACMP Polling Mode Configuration */
//    acmpConfig.pollingModeType      = ACMP_POSITIVE_POLLING;
//
//    acmpConfig.pollingSequence      = 7;
//    acmpConfig.pollingDiv           = ACMP_CLK_DIVIDE_256;
//    acmpConfig.hallOutEn            = ENABLE;
//    acmpConfig.hallAOutputCh        = ACMP_EXTERNAL_CH0;
//    acmpConfig.hallBOutputCh        = ACMP_EXTERNAL_CH1;
//    acmpConfig.hallCOutputCh        = ACMP_EXTERNAL_CH2;
//
//    /* ACMP Init Configuration */
//    acmpConfig.negativePin          = ACMP_EXTERNAL_CH3;
//    acmpConfig.edgeType             = ACMP_EDGE_TRIGGER_FALLING;
//    acmpConfig.interruptEn          = DISABLE;
//    acmpConfig.acmpEn               = ENABLE;
//
//    ACMP_Init(ACMP0, &acmpConfig);
//}

/*!
* @brief FOC Control Module PWDT Initialize.
*
* @param[in] none
* @return none
*/
void FOC_PWDT_Initialize(void)
{
    PWDT_ConfigType  pwdtConfig;

    /*
    * GPIO Configuration for PWDT1
    * PC8     --> PWDT1_IN0
    * PC7     --> PWDT1_IN1
    * PC6     --> PWDT1_IN2
    */
    /* PWDT channel 0 */
    GPIO_SetFunc(GPIOC, GPIO_PIN8, GPIO_FUN3);
    /* PWDT channel 1 */
    GPIO_SetFunc(GPIOC, GPIO_PIN7, GPIO_FUN3);
    /* PWDT channel 2 */
    GPIO_SetFunc(GPIOC, GPIO_PIN6, GPIO_FUN3);

    /* Config HALL Interface */
    memset(&pwdtConfig, 0, sizeof(pwdtConfig));
    pwdtConfig.overflowInterruptEn      = ENABLE;
    /* enable data reday interrupt */
    pwdtConfig.readyInterruptEn         = ENABLE;
    /* enable PWDT module interrupt */
    pwdtConfig.interruptEn              = ENABLE;
    /* enable PWDT module */
    pwdtConfig.mode                     = PWDT_MEASURE_MODE;
    /* start to measure pulse from first rising-edge and all dege */
    pwdtConfig.edgeType                 = PWDT_RISING_START_CAPTURE_ALL;
    /* PWDT clock source presclaer is 8 */
    pwdtConfig.clkPsc                   = (PWDT_ClkPrescalerType)PWDT_CLK_PRESCALER;

    pwdtConfig.cmpModeEn                = DISABLE;
    /* Enable PWDT Hall Interface */
    pwdtConfig.hallModeEn               = ENABLE;
    /* Enable PWDT Hall Filter value = 10KHz */
    pwdtConfig.filterEn                 = ENABLE;
    pwdtConfig.filterPsc                = (PWDT_FilterPrescalerType)PWDT_FILTER_PSC;
    pwdtConfig.filterValue              = PWDT_FILTVAL;
    pwdtConfig.callBack                 = FOC_PWDT_Callback;
    /* set PWDT channel */
    pwdtConfig.channel                  = PWDT_INPUT_CH_0;

    PWDT_Init(PWDT1, &pwdtConfig);
    NVIC_EnableIRQ(PWDT1_IRQn);
    /* Set interrupt priority */
    NVIC_SetPriority(PWDT1_IRQn, 1);
}

/*!
* @brief FOC Control Encoder Mode Module PWM Initialize.
*
* @param[in] none
* @return none
*/
void FOC_Encoder_Initialize(void)
{
    PWM_ConfigType pwmConfig;
    /* Encoder config struct */
    PWM_QuadDecodeConfigType encoderConfig;

    /*
    * GPIO Configuration for ENCODER
    * PA1     --> PWM0_CH0
    * PA0     --> PWM0_CH1
    * PB13    --> ENCODER_Z
    */
    /* Encoder phase A */
    GPIO_SetFunc(GPIOA, GPIO_PIN1, GPIO_FUN1);
    /* Encoder phase B */
    GPIO_SetFunc(GPIOA, GPIO_PIN0, GPIO_FUN1);
    /* Encoder Z gpio signal */
    GPIO_SetFunc(GPIOB, GPIO_PIN13, GPIO_FUN0);
    /* Encoder Z gpio input */
    GPIO_SetDir(GPIOB, GPIO_PIN13, GPIO_IN);

    encoderConfig.quadEn                        = ENABLE;
    /* AB phase encode,not cnt and dir encode */
    encoderConfig.mode                          = PWM_QUAD_PHASE_ENCODE;
    encoderConfig.phaseAConfig.filterEn         = ENABLE;
    encoderConfig.phaseAConfig.filterVal        = 5;
#if (ENCODER_POLARITY > 0)
    encoderConfig.phaseAConfig.polarity         = PWM_QUAD_PHASE_INVERT;
#else
    encoderConfig.phaseAConfig.polarity         = PWM_QUAD_PHASE_NORMAL;
#endif
    encoderConfig.phaseBConfig.filterEn         = ENABLE;
    encoderConfig.phaseBConfig.filterVal        = 5;
    encoderConfig.phaseBConfig.polarity         = PWM_QUAD_PHASE_NORMAL;

    pwmConfig.mode                              = PWM_MODE_QUADRATURE_DECODER;
    pwmConfig.initModeStruct                    = &encoderConfig;
    pwmConfig.clkPsc                            = 1;
    pwmConfig.initValue                         = 0;
    pwmConfig.maxValue                          = PULSE_PER_REVOLUTION - 1;
    pwmConfig.clkPsc = 0;
    pwmConfig.clkSource = PWM_CLK_SOURCE_APB;
    pwmConfig.overflowInterrupEn = ENABLE;
    pwmConfig.interruptEn = ENABLE;
    pwmConfig.callBack = FOC_Encoder_Callback;
    PWM_Init(PWM0, &pwmConfig);
#if (defined ABZ_QUAD_ENCODER)
    GPIO_EnableExtInterrupt(GPIOB, GPIO_PIN13, EXTI_TRIGGER_RISING);
    GPIO_SetCallback(GPIO_PIN13, GPIO_Z_callback);
#endif

    /* Set interrupt priority */
    NVIC_SetPriority(PWM0_IRQn, 2);
}

/*!
* @brief PWM Initialization for debugging, PWM module can be used for debugging output.
*
* @param[in] PWMx: PWM Module
* @return none
*/
void MC_DEBUG_PWM_Initialize(PWM_Type *PWMx)
{
    PWM_SimplyConfigType SimplyConfig;

    /*
    * GPIO Configuration for debug PWM
    * PB12    --> PWM0_CH2
    * PB11    --> PWM0_CH3
    */
    /* PWM0_CH2, Debug channel 0 */
    GPIO_SetFunc(GPIOB, GPIO_PIN12, GPIO_FUN1);
    /* PWM0_CH3, Debug channel 1 */
    GPIO_SetFunc(GPIOB, GPIO_PIN11, GPIO_FUN1);

    /* Config DEBUG-PWM Interface */
    memset(&SimplyConfig, 0, sizeof(SimplyConfig));
    SimplyConfig.allChCombineMode       = PWM_INDEPENDENT_MODE;
    SimplyConfig.countMode              = PWM_UP_COUNT;
    /* PWM_HIGH_TRUE */
    SimplyConfig.levelMode              = PWM_HIGH_TRUE;
    /* PWM_CLK_SOURCE_BUS */
    SimplyConfig.clkSource              = PWM_CLK_SOURCE_APB;
    SimplyConfig.clkPsc                 = 0;
    SimplyConfig.initValue              = 0;
    SimplyConfig.maxValue               = 4096;
    SimplyConfig.chValue[0]             = 0;
    SimplyConfig.chValue[2]             = 0;
    SimplyConfig.chValue[4]             = 0;
    SimplyConfig.interruptEn            = DISABLE;
    SimplyConfig.callBack               = NULL;
    PWM_SimplyInit(PWMx, &SimplyConfig);
}

/*!
* @brief SPI Initialization, SPI can be used when external DAC is uesed for debugging.
*
* @param[in] spiConfig: SPI Module Config Parameters
* @return none
*/
void MC_DEBUG_SPI_Initialize(SPI_ConfigType *spiConfig)
{
    DEVICE_ASSERT(spiConfig != NULL);

    /*
    * GPIO Configuration for SPI
    * PB0     --> MISO
    * PB1     --> CS
    * PB15    --> SCK
    * PB14    --> MOSI
    */

    /* MISO */
    GPIO_SetFunc(GPIOB, GPIO_PIN0, GPIO_FUN3);
    /* cs */
    GPIO_SetFunc(GPIOB, GPIO_PIN1, GPIO_FUN3);
    /* sck */
    GPIO_SetFunc(GPIOB, GPIO_PIN15, GPIO_FUN3);
    /* mosi */
    GPIO_SetFunc(GPIOB, GPIO_PIN14, GPIO_FUN3);

    spiConfig->mode = SPI_MASTER;
    spiConfig->interruptEn = DISABLE;
    spiConfig->spiEn = ENABLE;
    spiConfig->dmaRxEn = DISABLE;
    spiConfig->dmaTxEn = ENABLE;
    /* prescale is low + high + 2 */
    spiConfig->sckHigh = 2;
    /* baudrate = 48M / prescale  */
    spiConfig->sckLow = 2;

    /* Frame size is [3:15], stands for 4-16 bits */
    spiConfig->frmSize = SPI_FRAME_SIZE_16BITS;
    spiConfig->cpha = SPI_CPHA_1EDGE;
    spiConfig->cpol = SPI_CPOL_HIGH;
    spiConfig->txMsbFirstEn = ENABLE;
    spiConfig->rxMsbFirstEn = ENABLE;
    spiConfig->csOutputEn = ENABLE;
    spiConfig->continuousCSEn = DISABLE;

    SPI_Init(SPI1, spiConfig);
}

/*!
* @brief Motor control debug function initialization, SPI and PWM can be used for debugging output.
*
* @param[in] none
* @return none
*/
void MC_DEBUG_Initialize(void)
{
    if (DAC_PWM == ENABLE)
    {
        MC_DEBUG_PWM_Initialize(PWM0);
    }
    if (DAC_SPI == ENABLE)
    {
        MC_DEBUG_SPI_Initialize(&spiMasteConfig);
    }
}

/*!
* @brief FOC PWM Module Initialize.
*
* @param[in] none
* @return none
*/
void FOC_PWM_Initialize(void)
{
    PWM_DeInit(PWM1);
    PWM_CombineChConfig chConfig[4];
    PWM_ModulationConfigType pwmConfig;
    PWM_ConfigType config;
    PWM_FaultConfig faultConfig;
    PWM_SyncConfigType syncConfig;

    /*
    * GPIO Configuration for PWM1
    * PB5     --> PWM1_IN0
    * PB4     --> PWM1_IN1
    * PB8     --> PWM1_IN2
    * PB7     --> PWM1_IN3
    * PB10    --> PWM1_IN4
    * PB9     --> PWM1_IN5
    * PB6     --> PWM1_IN6
    */
    /* PWM output channel 0 */
    GPIO_SetFunc(GPIOB, GPIO_PIN5, GPIO_FUN1);
    /* PWM output channel 1 */
    GPIO_SetFunc(GPIOB, GPIO_PIN4, GPIO_FUN1);
    /* PWM output channel 2 */
    GPIO_SetFunc(GPIOB, GPIO_PIN8, GPIO_FUN1);
    /* PWM output channel 3 */
    GPIO_SetFunc(GPIOB, GPIO_PIN7, GPIO_FUN1);
    /* PWM output channel 4 */
    GPIO_SetFunc(GPIOB, GPIO_PIN10, GPIO_FUN1);
    /* PWM output channel 5 */
    GPIO_SetFunc(GPIOB, GPIO_PIN9, GPIO_FUN1);

    /* PWM hardware current overflow fault */
    GPIO_SetFunc(GPIOB, GPIO_PIN6, GPIO_FUN2);

    /* Config PWM Interface */
    memset(chConfig, 0, sizeof(chConfig));
    memset(&pwmConfig, 0, sizeof(PWM_ModulationConfigType));
    memset(&faultConfig, 0, sizeof(PWM_FaultConfig));
    memset(&syncConfig, 0, sizeof(PWM_SyncConfigType));
    memset(&config, 0, sizeof(PWM_ConfigType));
    CKGEN_Enable(CLK_PWM1, ENABLE);

    CKGEN_SoftReset(SRST_PWM1, ENABLE);
#if (defined GATE_DRIVER_HIGH_HIGH)
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_0, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_1, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_2, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_3, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_4, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_5, PWM_LOW_LEVEL);
#elif (defined GATE_DRIVER_HIGH_LOW)
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_0, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_1, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_2, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_3, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_4, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_5, PWM_HIGH_LEVEL);
#elif (defined GATE_DRIVER_LOW_HIGH)
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_0, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_1, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_2, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_3, PWM_LOW_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_4, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_5, PWM_LOW_LEVEL);
#elif (defined GATE_DRIVER_LOW_LOW)
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_0, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_1, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_2, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_3, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_4, PWM_HIGH_LEVEL);
    PWM_SetChannelOutputInitLevel(PWM1, PWM_CH_5, PWM_HIGH_LEVEL);
#endif
    PWM_SetInitChannelOutput(PWM1, ENABLE);

    /* channel 0/1 */
    chConfig[0].pairChannel                                     = PWM_CH_0;
    chConfig[0].ch1stValue                                      = 1;
    chConfig[0].ch2ndValue                                      = 1;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[0].levelMode                                       = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[0].complementEn                                    = ENABLE;
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[0].levelMode                                       = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* Nothing */
    #endif
    chConfig[0].complementEn                                    = DISABLE;
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[0].complementEn                                    = DISABLE;
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[0].complementEn                                    = ENABLE;
#endif
    chConfig[0].deadtimeEn                                      = ENABLE;

    chConfig[0].ch1stMatchDir                                   = PWM_MATCH_DIR_UP;
    chConfig[0].ch2ndMatchDir                                   = PWM_MATCH_DIR_DOWN;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[0].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[0].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[0].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[0].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[0].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* Nothing */
    #endif
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[0].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[0].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[0].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #endif
#endif

    /* channel 2/3 */
    chConfig[1].pairChannel                                     = PWM_CH_2;
    chConfig[1].ch1stValue                                      = 1;
    chConfig[1].ch2ndValue                                      = 1;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[1].levelMode                                       = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[1].complementEn                                    = ENABLE;
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[1].levelMode                                       = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* Nothing */
    #endif
    chConfig[1].complementEn                                    = DISABLE;
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[1].complementEn                                    = DISABLE;
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[1].complementEn                                    = ENABLE;
#endif
    chConfig[1].deadtimeEn                                      = ENABLE;

    chConfig[1].ch1stMatchDir                                   = PWM_MATCH_DIR_UP;
    chConfig[1].ch2ndMatchDir                                   = PWM_MATCH_DIR_DOWN;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[1].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[1].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[1].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[1].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[1].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* Nothing */
    #endif
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[1].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[1].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[1].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #endif
#endif

    /* channel 4/5 */
    chConfig[2].pairChannel                                     = PWM_CH_4;
    chConfig[2].ch1stValue                                      = 1;
    chConfig[2].ch2ndValue                                      = 1;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[2].levelMode                                       = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[2].complementEn                                    = ENABLE;
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[2].levelMode                                       = PWM_HIGH_TRUE;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* Nothing */
    #endif
    chConfig[2].complementEn                                    = DISABLE;
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[2].complementEn                                    = DISABLE;
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2].levelMode                                       = PWM_LOW_TRUE;
    #endif
    chConfig[2].complementEn                                    = ENABLE;
#endif
    chConfig[2].deadtimeEn                                      = ENABLE;

    chConfig[2].ch1stMatchDir                                   = PWM_MATCH_DIR_UP;
    chConfig[2].ch2ndMatchDir                                   = PWM_MATCH_DIR_DOWN;
#if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[2].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[2].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[2].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    chConfig[2].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    chConfig[2].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* Nothing */
    #endif
#elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[2].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_HIGH;
    #endif
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[2].ch1stPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    chConfig[2].ch2ndPolarity                                   = PWM_OUTPUT_POLARITY_ACTIVE_LOW;
    #endif
#endif

#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    /* channel 6/7 */
    chConfig[3].pairChannel                                     = PWM_CH_6;
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* 0 means Up edge sample */
    #if (SAMPLE_TRIG_EDGE == 0)
    {
        chConfig[3].ch1stValue                                  = PWM_PERIOD_VALUE >> 2;
        chConfig[3].ch2ndValue                                  = (PWM_PERIOD_VALUE >> 1);
    }
    /* 1 means Down edge sample */
    #else
    {
        chConfig[3].ch1stValue                                  = PWM_PERIOD_VALUE >> 1;
        chConfig[3].ch2ndValue                                  = (PWM_PERIOD_VALUE >> 2);
    }
    #endif

    #else
    chConfig[3].ch1stValue                                      = (PWM_PERIOD_VALUE >> 4);
    chConfig[3].ch2ndValue                                      = (PWM_PERIOD_VALUE >> 4) + (PWM_PERIOD_VALUE >> 1);
    #endif
    #else

    #endif

    chConfig[3].levelMode                                       = PWM_HIGH_TRUE;
    chConfig[3].deadtimeEn                                      = DISABLE;
    chConfig[3].complementEn                                    = DISABLE;
    /* 0 means Up edge sample */
    #if (SAMPLE_TRIG_EDGE == 0)
    {
        chConfig[3].ch1stMatchDir                               = PWM_MATCH_DIR_UP;
        chConfig[3].ch2ndMatchDir                               = PWM_MATCH_DIR_UP;
    }
    /* 1 means Down edge sample */
    #else
    {
        chConfig[3].ch1stMatchDir                               = PWM_MATCH_DIR_DOWN;
        chConfig[3].ch2ndMatchDir                               = PWM_MATCH_DIR_DOWN;
    }
    #endif

    chConfig[3].ch1stTriggerEn                                  = ENABLE;
    chConfig[3].ch2ndTriggerEn                                  = ENABLE;
#elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    #if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    chConfig[3].pairChannel                                     = PWM_CH_6;
    chConfig[3].ch1stValue                                      = PWM_PERIOD_VALUE;
    chConfig[3].ch2ndValue                                      = 0;
    chConfig[3].levelMode                                       = PWM_HIGH_TRUE;
    chConfig[3].deadtimeEn                                      = DISABLE;
    chConfig[3].complementEn                                    = DISABLE;
    chConfig[3].ch1stMatchDir                                   = PWM_MATCH_DIR_UP;
    chConfig[3].ch2ndMatchDir                                   = PWM_MATCH_DIR_UP;
    chConfig[3].ch1stTriggerEn                                  = ENABLE;
    chConfig[3].ch2ndTriggerEn                                  = DISABLE;
    #endif

    #elif (defined GATE_DRIVER_HIGH_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    #endif

    #elif (defined GATE_DRIVER_LOW_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    /* Nothing */
    #endif

    #elif (defined GATE_DRIVER_LOW_LOW)
    #endif

#elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
    chConfig[3].pairChannel                                     = PWM_CH_6;
    chConfig[3].ch1stValue                                      = 1;
    chConfig[3].levelMode                                       = PWM_HIGH_TRUE;
    chConfig[3].deadtimeEn                                      = DISABLE;
    chConfig[3].complementEn                                    = DISABLE;
    chConfig[3].ch1stMatchDir                                   = PWM_MATCH_DIR_DOWN;
    chConfig[3].ch1stTriggerEn                                  = ENABLE;
#endif

    /* modulation mode config */
#if ((CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE))
    pwmConfig.initTriggerEn                                     = DISABLE;
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    pwmConfig.countMode                                         = PWM_UP_DOWN_COUNT;
    #else
    pwmConfig.countMode                                         = PWM_UP_COUNT;
    #endif
    pwmConfig.combineChannelNum                                 = 4;
    pwmConfig.combineChConfig                                   = chConfig;
    pwmConfig.deadtime                                          = PWM_DEAD_TIME;
    pwmConfig.deadtimePsc                                       = PWM_DEADTIME_DIVID_4;

#elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    #if (defined GATE_DRIVER_HIGH_HIGH)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    pwmConfig.initTriggerEn                                     = ENABLE;
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    pwmConfig.countMode                                         = PWM_UP_DOWN_COUNT;
    #else
    pwmConfig.countMode                                         = PWM_UP_COUNT;
    #endif
    pwmConfig.combineChannelNum                                 = 3;
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    pwmConfig.initTriggerEn                                     = DISABLE;
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    pwmConfig.countMode                                         = PWM_UP_DOWN_COUNT;
    #else
    pwmConfig.countMode                                         = PWM_UP_COUNT;
    #endif
    pwmConfig.combineChannelNum                                 = 4;
    #endif

    #elif (defined GATE_DRIVER_HIGH_LOW)
    pwmConfig.initTriggerEn                                     = ENABLE;
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    pwmConfig.countMode                                         = PWM_UP_DOWN_COUNT;
    #else
    pwmConfig.countMode                                         = PWM_UP_COUNT;
    #endif
    pwmConfig.combineChannelNum                                 = 3;

#elif (defined GATE_DRIVER_LOW_HIGH)
    pwmConfig.initTriggerEn                                     = ENABLE;
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    pwmConfig.countMode                                         = PWM_UP_DOWN_COUNT;
    #else
    pwmConfig.countMode                                         = PWM_UP_COUNT;
    #endif
    pwmConfig.combineChannelNum                                 = 3;

#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    /* Nothing */
    #elif (PHASE_UVW_POLARITY == ACTIVE_LOW)
    pwmConfig.initTriggerEn                                     = ENABLE;
    #if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
    pwmConfig.countMode                                         = PWM_UP_DOWN_COUNT;
    #else
    pwmConfig.countMode                                         = PWM_UP_COUNT;
    #endif
    pwmConfig.combineChannelNum                                 = 3;
    #endif
    #endif

    pwmConfig.combineChConfig                                   = chConfig;
    pwmConfig.deadtime                                          = PWM_DEAD_TIME;
    pwmConfig.deadtimePsc                                       = PWM_DEADTIME_DIVID_4;
#endif

    /* pwm config */
    config.mode                                                 = PWM_MODE_MODULATION;
    config.initModeStruct                                       = &pwmConfig;
    config.clkSource                                            = PWM_CLK_SOURCE_NONE;
    config.clkPsc                                               = PWM_PRSC;
    config.maxValue                                             = PWM_PERIOD_VALUE;
    config.interruptEn                                          = ENABLE;
    config.callBack                                             = FOC_PWM_FaultCallback;
    PWM_Init(PWM1, &config);

    /* pwm sync config */
    syncConfig.syncTriggerMode                                  = PWM_SYNC_TRIGGER_SOFTWARE;
    syncConfig.hardwareSync0En                                  = DISABLE;
    syncConfig.hardwareSync1En                                  = DISABLE;
    syncConfig.hardwareSync2En                                  = DISABLE;
    syncConfig.autoClearHWTriggerEn                             = DISABLE;
    syncConfig.counterInitSyncEn                                = ENABLE;
    syncConfig.outCtrlSyncEn                                    = DISABLE;
    syncConfig.inverterSyncEn                                   = DISABLE;
    syncConfig.outmaskSyncEn                                    = DISABLE;
    syncConfig.polaritySyncEn                                   = DISABLE;
    syncConfig.chValueSyncEn[0]                                 = ENABLE;
    syncConfig.chValueSyncEn[1]                                 = ENABLE;
    syncConfig.chValueSyncEn[2]                                 = ENABLE;
    syncConfig.chValueSyncEn[3]                                 = ENABLE;
    syncConfig.maxLoadingPointEn                                = DISABLE;
    syncConfig.minLoadingPointEn                                = ENABLE;
    syncConfig.syncEn = ENABLE;
    PWM_InitSyncControl(PWM1, &syncConfig);

    /* fault control config */
    faultConfig.channelConfig[PWM_FAULT_CH_1].faultInputEn      = ENABLE;
    faultConfig.channelConfig[PWM_FAULT_CH_1].faultFilterEn     = DISABLE;
    faultConfig.channelConfig[PWM_FAULT_CH_1].faultPolarity     = OVERCUR_FEEDBACK_POLARITY;
    faultConfig.mode                                            = PWM_FAULT_CTRL_MANUAL_ALL;
    faultConfig.filterValue                                     = 0;
    faultConfig.faultCtrlOutputEn[0]                            = ENABLE;
    faultConfig.faultCtrlOutputEn[1]                            = ENABLE;
    faultConfig.faultCtrlOutputEn[2]                            = ENABLE;
    faultConfig.faultCtrlOutputEn[3]                            = ENABLE;
    faultConfig.interruptEn                                     = ENABLE;
    PWM_InitFaultControl(PWM1, &faultConfig);

    /* Set interrupt priority */
    NVIC_SetPriority(PWM1_IRQn, 0);
    /* PWM Start */
    PWM_SetClockSource(PWM1, PWM_CLK_SOURCE_APB);
}

/*!
* @brief FOC_TIM_Initialize(timer Initialize), generates timer interrupt.
*
* @param[in] none
* @return none
*/
void FOC_TIM_Initialize(void)
{
    TIMER_ConfigType timerConfig = {0};
    memset(&timerConfig, 0, sizeof(TIMER_ConfigType));
    /* config 500us delay to do pid calculation */
    timerConfig.periodValue             = TIMER_1MS;
    timerConfig.interruptEn             = ENABLE;
    timerConfig.linkModeEn              = DISABLE;
    timerConfig.timerEn                 = ENABLE;

    timerConfig.callBack                = Timer0_Callback;
    TIMER_Init(TIMER_CHANNEL0, &timerConfig);
    /* Set interrupt priority */
    NVIC_SetPriority(TIMER_CHANNEL0_IRQn, 3);
}

/*!
* @brief Encoder_TIM_Initialize(timer Initialize), generates timer interrupt for M_T speed measure.
*
* @param[in] none
* @return none
*/
void Encoder_TIM_Initialize(void)
{
    TIMER_ConfigType timerConfig = {0};
    memset(&timerConfig, 0, sizeof(TIMER_ConfigType));
    /* config 500us delay to do pid calculation */
    timerConfig.periodValue             = M_T_PERIOD;
    timerConfig.interruptEn             = ENABLE;
    timerConfig.linkModeEn              = DISABLE;
    timerConfig.timerEn                 = ENABLE;

    timerConfig.callBack                = Timer1_Callback;
    TIMER_Init(TIMER_CHANNEL1, &timerConfig);
    /* Set interrupt priority */
    NVIC_SetPriority(TIMER_CHANNEL1_IRQn, 1);
}

/*!
* @brief CTU Module Initialize.
*
* @param[in] none
* @return none
*/
void FOC_CTU_Initialize(void)
{
    CTU_DeInit();
    CTU_ConfigType ctuConfig;
    memset(&ctuConfig, 0, sizeof(CTU_ConfigType));
    ctuConfig.clkPsc                    = CTU_CLK_PRESCALER_1;

    ctuConfig.adcInjectTriggerSource    = CTU_ADC_TRIGGER_SOURCE;

    ctuConfig.delay1Time                = 0;

    CTU_Init(&ctuConfig);
}

/*!
* @brief DMA Module Initialize.
*
* @param[in] none
* @return none
*/
void FOC_DMA_Init()
{
    DMA_ChannelType *DMAx                       = DMA0_CHANNEL0;
    uint32_t tmpMemStartAddr                    = (uint32_t)&g_ADCRegularBuffer[0];
    /* Setting memory DMA address */
    uint32_t tmpMemEndAddr                      = (uint32_t)&g_ADCRegularBuffer[7];
    DMA_ConfigType tmpDMAConfig;
    memset(&tmpDMAConfig, 0x00, sizeof(DMA_ConfigType));

    tmpDMAConfig.memStartAddr                   = tmpMemStartAddr;
    tmpDMAConfig.memEndAddr                     = tmpMemEndAddr;
    /* Move ADC DR to memory */
    tmpDMAConfig.periphStartAddr                = (uint32_t)(&(ADC0->RDR));
    /* Enable DMAx channel */
    tmpDMAConfig.channelEn                      = ENABLE;
    /* Enable DMA transfer complete interrupt */
    tmpDMAConfig.finishInterruptEn              = DISABLE;
    /* Disable DMA transfer half complete interrupt */
    tmpDMAConfig.halfFinishInterruptEn          = DISABLE;
    tmpDMAConfig.errorInterruptEn               = DISABLE;
    /* 3: Very high priority */
    tmpDMAConfig.channelPriority                = DMA_PRIORITY_VERY_HIGH;
    /* Circular mode enable,If want work one time disable */
    tmpDMAConfig.circular                       = ENABLE;
    /* 1: Read from peripheral */
    tmpDMAConfig.direction                      = DMA_READ_FROM_PERIPH;
    /* Peripheral to memory */
    tmpDMAConfig.MEM2MEM                        = DISABLE;
    /* 1:16-bit[15:0]; 2:[23:16],[7:0] */
    tmpDMAConfig.memByteMode                    = DMA_MEM_BYTE_MODE_1TIME;
    /* Memory address increment */
    tmpDMAConfig.memIncrement                   = ENABLE;
    /* Peripheral address fixed */
    tmpDMAConfig.periphIncrement                = DISABLE;
    /* 2:32-bit */
    tmpDMAConfig.memSize                        = DMA_MEM_SIZE_32BIT;
    /* Fixed 1:16-bit */
    tmpDMAConfig.periphSize                     = DMA_PERIPH_SIZE_16BIT;
    /* As request */
    tmpDMAConfig.transferNum                    = 6;
    tmpDMAConfig.periphSelect                   = DMA_PEPIRH_ADC0;

    /* Use DMA1 channel */
    DMA_Init(DMAx, &tmpDMAConfig);

}

/*!
* @brief Calbiration ADC to get adc channel offset value.
*
* @param[in] sequencechannel : ADC Inject Group sequence num, 0-3
* @return dcoffset: ADC Inject Group channel offset
*/
int16_t MC_ADC_Calbiration(uint8_t sequencechannel)
{
    uint16_t index = 0;
    uint32_t sumOffset =0;
    int16_t adcOffset = 0;

    ADC_SetRegularTriggerSource(ADC0, ADC_TRIGGER_INTERNAL);
    ADC_SetInjectTriggerSource(ADC0, ADC_TRIGGER_INTERNAL);

    ADC_ClearIEOCFlag(ADC0);
    udelay(10);

    for (index = 0; index < NUM_OF_CONVERSIONS; index ++)
    {
        ADC_SoftwareStartInjectConvert(ADC0);
#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
        udelay(10);
        ADC_SoftwareStartInjectConvert(ADC0);
#endif
        while (!ADC_GetIEOCFlag(ADC0));

        sumOffset += ADC_GetInjectData(ADC0, sequencechannel);

        ADC_ClearIEOCFlag(ADC0);
    }
    adcOffset = sumOffset / NUM_OF_CONVERSIONS;

    return adcOffset;
}

/*!
* @brief Get phase current sampling offset value.
*
* @param[in] adcSampleReal: pointer to ADC_SAMPLE structure
* @return none
*/
void FOC_ADC_Calbiration(ADC_SAMPLE *adcSampleReal)
{
#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    adcSampleReal->ibusOffset = MC_ADC_Calbiration(0);
#elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    adcSampleReal->iaOffset = MC_ADC_Calbiration(1);
    adcSampleReal->ibOffset = MC_ADC_Calbiration(2);
#elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
    adcSampleReal->iaOffset = MC_ADC_Calbiration(0);
    adcSampleReal->ibOffset = MC_ADC_Calbiration(1);
    adcSampleReal->icOffset = MC_ADC_Calbiration(2);
#else

#endif
}

/*!
* @brief ADC Initialize, first calibration current Offset, then config adc as external trigger mode.
*
* @param[in] none
* @return none
*/
void FOC_ADC_Initialize(void)
{
    ADC_DeInit(ADC0);
    ADC_ConfigType tempAdcConfig;
    ADC_ConfigType* adcConfig;
    adcConfig = &tempAdcConfig;
    memset(adcConfig, 0x00, sizeof(ADC_ConfigType));

    /*
    * GPIO Configuration for ADC
    * PA7     --> ADC_IN4
    * PA5     --> ADC_IN5
    * PA4     --> ADC_IN6
    * PA3     --> ADC_IN7
    * PA2     --> ADC_IN8
    * PC1     --> ADC_IN9
    * PC0     --> ADC_IN10
    */

    /* ADC channel 8, V_Bus */
    GPIO_SetFunc(GPIOA, GPIO_PIN2, GPIO_FUN2);
    /* ADC channel 9, VARIOHM_SPEED_ADJUST */
    GPIO_SetFunc(GPIOC, GPIO_PIN1, GPIO_FUN2);
    /* ADC channel 10, Control keys(Star_Stop/Fast/Slow/Direction) */
    GPIO_SetFunc(GPIOC, GPIO_PIN0, GPIO_FUN2);

    /* ACMP_in0, Back-Emf of Phase U */
    GPIO_SetFunc(GPIOA, GPIO_PIN11, GPIO_FUN2);
    /* ACMP_in1, Back-Emf of Phase V */
    GPIO_SetFunc(GPIOA, GPIO_PIN10, GPIO_FUN2);
    /* ACMP_in2, Back-Emf of Phase W */
    GPIO_SetFunc(GPIOA, GPIO_PIN9, GPIO_FUN2);

#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    /* ADC channel 7, I_Bus */
    GPIO_SetFunc(GPIOA, GPIO_PIN3, GPIO_FUN2);
#elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    /* ADC channel 4, I_U */
    GPIO_SetFunc(GPIOA, GPIO_PIN7, GPIO_FUN2);
    /* ADC channel 5, I_V */
    GPIO_SetFunc(GPIOA, GPIO_PIN5, GPIO_FUN2);
    /* ADC channel 7, I_Bus */
    GPIO_SetFunc(GPIOA, GPIO_PIN3, GPIO_FUN2);
#elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
    /* ADC channel 4, I_U */
    GPIO_SetFunc(GPIOA, GPIO_PIN7, GPIO_FUN2);
    /* ADC channel 5, I_V */
    GPIO_SetFunc(GPIOA, GPIO_PIN5, GPIO_FUN2);
    /* ADC channel 6, I_W */
    GPIO_SetFunc(GPIOA, GPIO_PIN4, GPIO_FUN2);
    /* ADC channel 7, I_Bus */
    GPIO_SetFunc(GPIOA, GPIO_PIN3, GPIO_FUN2);
#else

#endif
    /* ADC channel 0, Bemf_U */
    GPIO_SetFunc(GPIOA, GPIO_PIN11, GPIO_FUN2);
    /* ADC channel 1, Bemf_V */
    GPIO_SetFunc(GPIOA, GPIO_PIN10, GPIO_FUN2);
    /* ADC channel 2, Bemf_W */
    GPIO_SetFunc(GPIOA, GPIO_PIN9, GPIO_FUN2);

    CKGEN_Enable(CLK_ADC0, ENABLE);

    adcConfig->scanModeEn = ENABLE;
    /* ADC mode 3 */
    adcConfig->continousModeEn = DISABLE;
    adcConfig->regularDiscontinousModeEn = DISABLE;
    adcConfig->injectDiscontinousModeEn = DISABLE;
    adcConfig->injectAutoModeEn = DISABLE;
#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    adcConfig->intervalModeEn = ENABLE;
#elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    adcConfig->intervalModeEn = DISABLE;

#else

#endif

    #if (defined SYS_CLK_72M)
    adcConfig->clkPsc = ADC_CLK_PRESCALER_2;
    #elif (defined SYS_CLK_48M)
    adcConfig->clkPsc = ADC_CLK_PRESCALER_1;
    #endif

    adcConfig->regularSequenceLength = 3;
#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    adcConfig->injectSequenceLength = 2;
#elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    adcConfig->injectSequenceLength = 3;
#elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
    adcConfig->injectSequenceLength = 4;
#else

#endif

    adcConfig->regularDiscontinousNum = 0;
    adcConfig->EOCInterruptEn = DISABLE;
    adcConfig->IEOCInterruptEn = DISABLE;
    adcConfig->interruptEn = DISABLE;
    adcConfig->regularDMAEn = ENABLE;
    /* set adc isr callback path */
    adcConfig->callBack = Motor_Ctrl_CallBack;
    adcConfig->dataAlign = ADC_DATA_ALIGN_RIGHT;
    adcConfig->powerMode = ADC_POWER_ON;
    adcConfig->regularTriggerMode = ADC_TRIGGER_INTERNAL;
    adcConfig->injectTriggerMode = ADC_TRIGGER_INTERNAL;
    /* ADC works Channel Config */
    ADC_Init(ADC0, adcConfig);

    /* VBUS */
    ADC_SetRegularGroupChannel(ADC0, ADC_CH_8, AD_SAMP_CLK, 0);
    /* ADC SPEED */
    ADC_SetRegularGroupChannel(ADC0, ADC_CH_9, AD_SAMP_CLK, 1);
    /* key */
    ADC_SetRegularGroupChannel(ADC0, ADC_CH_10, AD_SAMP_CLK, 2);

#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    /* IBUS */
    ADC_SetInjectGroupChannel(ADC0, ADC_CH_7, AD_SAMP_CLK, 0);
    /* IBUS */
    ADC_SetInjectGroupChannel(ADC0, ADC_CH_7, AD_SAMP_CLK, 1);
#elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    /* IBUS */
    ADC_SetInjectGroupChannel(ADC0, ADC_CH_7, AD_SAMP_CLK, 0);
    /* IU */
    ADC_SetInjectGroupChannel(ADC0, ADC_CH_4, AD_SAMP_CLK, 1);
    /* IV */
    ADC_SetInjectGroupChannel(ADC0, ADC_CH_5, AD_SAMP_CLK, 2);
#elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
    /* IU */
    ADC_SetInjectGroupChannel(ADC0, ADC_CH_4, AD_SAMP_CLK, 0);
    /* IV */
    ADC_SetInjectGroupChannel(ADC0, ADC_CH_5, AD_SAMP_CLK, 1);
    /* IW */
    ADC_SetInjectGroupChannel(ADC0, ADC_CH_6, AD_SAMP_CLK, 2);
    /* IBUS */
    ADC_SetInjectGroupChannel(ADC0, ADC_CH_7, AD_SAMP_CLK, 3);

#else

#endif

    /* Calbiration Ia,Ib,Ic Offset */
    FOC_ADC_Calbiration(&g_adcSampleReal);

    /* Set SVPWM ADC to external trigger source */
#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    ADC_DeInit(ADC0);
    /* ADC works Channel Config */
    ADC_Init(ADC0, adcConfig);

    /* VBUS */
    ADC_SetRegularGroupChannel(ADC0, ADC_CH_8, AD_SAMP_CLK, 0);
    /* ADC SPEED */
    ADC_SetRegularGroupChannel(ADC0, ADC_CH_9, AD_SAMP_CLK, 1);
    /* key */
    ADC_SetRegularGroupChannel(ADC0, ADC_CH_10, AD_SAMP_CLK, 2);

    /* IBUS */
    ADC_SetInjectGroupChannel(ADC0, ADC_CH_7, AD_SAMP_CLK, 0);
    /* IBUS */
    ADC_SetInjectGroupChannel(ADC0, ADC_CH_7, AD_SAMP_CLK, 1);
#endif
    ADC_SetInjectTriggerSource(ADC0, ADC_TRIGGER_EXTERNAL);
    ADC_SetRegularTriggerSource(ADC0, ADC_TRIGGER_INTERNAL);
#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    ADC_ClearIEOCFlag(ADC0);
#endif
#if (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
    ADC_SetInjectLength(ADC0, 3);
#endif
    ADC_SetIEOCInterrupt(ADC0, ENABLE);

    FOC_DMA_Init();
    /* Start adc convert */
    ADC_SoftwareStartRegularConvert(ADC0);

    NVIC_SetPriority(ADC0_IRQn, 1);

}

/*!
* @brief Peripheral Initialize in foc App.
*
* @param[in] none
* @return none
*/
void FOC_Peripheral_Initialize(void)
{
    FOC_GPIO_Initialize();
    FOC_TIM_Initialize();
    FOC_CTU_Initialize();
    FOC_ADC_Initialize();
//    SpeedControlByPWM_init();

    FOC_PWM_Initialize();
    PWM_Output(PWM_OUTPUT_DISABLE);

#if (defined HALL_SENSORS)
    /* pwdt initialize should before HallSensor_Init */
    FOC_PWDT_Initialize();
#endif
#if (defined SENSOR_ACIM)
    FOC_Encoder_Initialize();
    #if (defined MT_SPEED_MEASURE)
    Encoder_TIM_Initialize();
    #endif
#elif (defined SENSORLESS_ACIM)
#else
#endif

#if (defined ENCODER_SENSORS)
    FOC_Encoder_Initialize();
    #if (defined MT_SPEED_MEASURE)
    Encoder_TIM_Initialize();
    #endif
#elif (defined ENCODER_HALL_SENSORS)
    FOC_PWDT_Initialize();
    FOC_Encoder_Initialize();
    #if (defined MT_SPEED_MEASURE)
    Encoder_TIM_Initialize();
    #endif
#else
#if (defined DAC_FUNCTIONALITY)
    MC_DEBUG_Initialize();
#endif
#endif

    NVIC_EnableIRQ(ADC0_IRQn);
}

/* =============================================  EOF  ============================================== */
